@@ -48,8 +48,8 @@ def local(command_string):
 class FabricException(Exception):
     pass
 
-TRUTH_VALUES = ['True', 'TRUE', '1', 'true', 't', 'Yes',
-                'YES', 'yes', 'y']  # arguments in fab are always strings
+TRUTH_VALUES = ['True', 'TRUE', '1', 'true', 't', 'Yes','YES', 'yes', 'y']  # arguments in fab are always strings
+FALSE_VALUES = ['False', 'FALSE', '0', 'false', 'f', 'No', 'NO', 'no', 'n']
 
 # Set some global defaults for all operations.
 # These ones may be modified by an ENV var.
@@ -289,7 +289,7 @@ def _log_hosts(instances):
 
 
 @task
-def build(branch, installPath, node=False, angularBuild=False):
+def build(branch, ngBuild='', ngApp='', angularBuild='', angularEnv='', installPath='.', node=''):
 
 
     # ensure yarn installs all build tools
@@ -297,18 +297,22 @@ def build(branch, installPath, node=False, angularBuild=False):
         local("pwd")
         local('yarn --production=false --no-progress --non-interactive install')
 
-        # do the build
-        if angularBuild:
-            local(
-                'npm run build -- --prod --configuration=%s && cp -r public/* dist/' % ENVIRONMENT)
-        else:
-            local('npm run dist')
+ 
+        if ngBuild:
+            options = ' --configuration=%s ' % angularEnv if angularEnv else ''
+            local('yarn build %s %s' % (options, ngApp))
+            local('cp {package.json,yarn.lock,knexfile.js} dist/')
+            local('cp -R migrations/ dist/migrations')
+            local('if [ -d "apps/%s/config" ]; then cp -R "apps/%s/config" "dist/apps/%s/"; fi' % (ngApp,ngApp,ngApp))
 
-        if node in TRUTH_VALUES:
-            local(
-                'if [[ -d "config" ]]; then echo "<collecting config>" ; rsync -ra --stats config/ dist/config; fi;')
-            local(
-                'if [[ -d "src/public" ]]; then echo "<collecting public>" ; rsync -ra --stats src/public/ dist/public; fi;')
+        elif angularBuild:
+            config = angularEnv if angularEnv else ENVIRONMENT
+            local('npm run build -- --prod --configuration=%s && cp -r public/* dist/' % config)
+
+        elif node:
+            local('npm run dist')
+            local('if [[ -d "config" ]]; then echo "<collecting config>" ; rsync -ra --stats config/ dist/config; fi;')
+            local('if [[ -d "src/public" ]]; then echo "<collecting public>" ; rsync -ra --stats src/public/ dist/public; fi;')
 
 
     # make sure the new files are part of the local git repo
@@ -387,11 +391,13 @@ def pip_install():
 
 
 @task
-def yarn_install(installPath):
+def yarn_install(installPath=''):
+
+    fullPath = '/data/deploy/pending/%s' % installPath
 
     try:
-        with cd("/data/deploy/pending/%s" % installPath):
-            sudo('yarn --no-progress --non-interactive install')
+        with cd(fullPath):
+            sudo('yarn --production=true --no-progress --non-interactive install')
     except FabricException:
         pass
 
@@ -441,7 +447,7 @@ def codeversioner():
 # This assumes the local repo is ready (any build has been done and
 # commited), then creates the tarball, finally deploys one at a time
 @task
-def deploycode(branch, nltkLoad="False", doCollectStatic="True", yarn="False", immutable="True"):
+def deploycode(branch, nltkLoad="False", doCollectStatic="True", djangoAdmin="True", yarn="False", immutable="True"):
     tar_from_git(branch)
     remote_inflate_code()
 
@@ -456,7 +462,7 @@ def deploycode(branch, nltkLoad="False", doCollectStatic="True", yarn="False", i
     if doCollectStatic in TRUTH_VALUES:
         collect_static()
     else:
-        if not yarn in TRUTH_VALUES:
+        if (not yarn in TRUTH_VALUES) and (djangoAdmin in TRUTH_VALUES):
             sudo('cp -r ' + '/usr/local/opt/python/lib/python2.7/site-packages' +
                  '/django/contrib/admin/static/admin /data/deploy/pending/static/')
 
@@ -469,7 +475,7 @@ def deploycode(branch, nltkLoad="False", doCollectStatic="True", yarn="False", i
 # done prior), then deploys all targets in parallel
 @task
 @parallel
-def deployParallel(nltkLoad="False", doCollectStatic="True", yarn="False", immutable="True"):
+def deployParallel(nltkLoad="False", doCollectStatic="True", djangoAdmin="True", yarn="False", immutable="True"):
     remote_inflate_code()
 
     if not yarn in TRUTH_VALUES:
@@ -483,7 +489,7 @@ def deployParallel(nltkLoad="False", doCollectStatic="True", yarn="False", immut
     if doCollectStatic in TRUTH_VALUES:
         collect_static()
     else:
-        if not yarn in TRUTH_VALUES:
+        if (not yarn in TRUTH_VALUES) and (djangoAdmin in TRUTH_VALUES):
             sudo('cp -r ' + '/usr/local/opt/python/lib/python2.7/site-packages' +
                  '/django/contrib/admin/static/admin /data/deploy/pending/static/')
 
@@ -493,9 +499,9 @@ def deployParallel(nltkLoad="False", doCollectStatic="True", yarn="False", immut
 
 
 @task
-def dbmigrate_node(installPath):
+def dbmigrate_node(installPath=''):
 
-    pathToUse = '/data/deploy/pending/' + installPath + "/dist"
+    pathToUse = '/data/deploy/pending/' + installPath + '/dist' if installPath else '/data/deploy/pending/dist'
 
     with cd(pathToUse):
         run('pwd')
@@ -539,23 +545,25 @@ supervisor = "/usr/bin/supervisorctl"
 
 @task
 @parallel
-def update_app_utils(appname, branch, reboot=False):
+def update_app_utils(appname, branch, keys=False, reboot=False):
     # tar up the app-util code with only the environment and config directory
     local("pwd")
-    # need to check to see if they are using dcShared-utils
+
+    # If using a shared utils repo, then need to move into this app's utils folder.
     APP_UTILS_DIR = appname + "-utils"
-    if not os.path.isdir(APP_UTILS_DIR):
-        # it's not in the current directory so it must be under the dcShared-utils directory
-        lcd("dcShared-utils")
+    if os.path.isdir(APP_UTILS_DIR):
+        lcd(APP_UTILS_DIR)
     
     # tar it up
     TAR_NAME = "app-utils"
     CONFIG_DIR = APP_UTILS_DIR + "/config"
     ENV_DIR = APP_UTILS_DIR + "/environments"
-    DIRS_TO_TAR = CONFIG_DIR + " " + ENV_DIR
+    KEY_DIR = (APP_UTILS_DIR + "/keys/" + ENVIRONMENT) if (keys in TRUTH_VALUES) else ""
+
+    DIRS_TO_TAR = CONFIG_DIR + " " + ENV_DIR + " " + KEY_DIR
     print(DIRS_TO_TAR)
 
-    local('git archive %s --format=tar.gz --output=%s.tar.gz %s %s' % (branch, TAR_NAME, CONFIG_DIR, ENV_DIR))
+    local('git archive %s --format=tar.gz --output=%s.tar.gz %s %s %s' % (branch, TAR_NAME, CONFIG_DIR, ENV_DIR, KEY_DIR))
 
     # and move the tarball to the destination and expand it
     # set the upload path to the home directory
@@ -595,11 +603,10 @@ def rollback_code():
         sudo('unlink /data/deploy/pending')
 
     with cd("/data/deploy/"):
-
-        run('(ls -t|grep 20|head -2|ls|grep 20)|sort -r|head -2|tail -1|xargs -I % sudo ln -s /data/deploy/% /data/deploy/current')
-        run("(ls -t|grep 20|head -2|ls|grep 20)|sort -r|head -2|tail -1|xargs -I \\% sudo ln -s /data/deploy/\\% /data/deploy/pending")
-
-#        run("ls -t|grep 20|head -2|ls|grep 20|sort -r|head -1 |xargs -I \\% sudo rm -rf \\%")  
+        # Point to the previous deploy and delete the one that we're discarding.
+        run("(ls -t|grep 20|head -2|ls|grep 20)|sort -r|head -2|tail -1|xargs -I % sudo ln -s /data/deploy/% /data/deploy/current")
+        run("(ls -t|grep 20|head -2|ls|grep 20)|sort -r|head -2|tail -1|xargs -I % sudo ln -s /data/deploy/% /data/deploy/pending")
+        run("ls -t|grep 20|head -2|ls|grep 20|sort -r|head -1 |xargs sudo rm -rf")  
 
 
 @task
